@@ -1,5 +1,6 @@
-# -*- coding: utf-8 -*-
 from lxml import etree
+from plone.base.utils import safe_bytes
+from plone.base.utils import safe_text
 from plone.registry import FieldRef
 from plone.registry import Record
 from plone.registry.interfaces import IFieldRef
@@ -20,6 +21,7 @@ from zope.configuration import xmlconfig
 from zope.dottedname.resolve import resolve
 from zope.schema import getFieldNames
 
+
 _marker = object()
 
 
@@ -35,6 +37,7 @@ def evaluateCondition(expression):
     """
     try:
         import Zope2.App.zcml
+
         context = Zope2.App.zcml._context or config.ConfigurationMachine()
     except ImportError:
         context = config.ConfigurationMachine()
@@ -43,12 +46,11 @@ def evaluateCondition(expression):
     return handler.evaluateCondition(expression)
 
 
-
 def shouldPurgeList(value_node, key):
     for child in value_node:
         attrib = child.attrib
-        if attrib.get('key') == key:
-            if attrib.get('purge', 'true').lower() == 'false':
+        if attrib.get("key") == key:
+            if attrib.get("purge", "true").lower() == "false":
                 return False
             else:
                 return True
@@ -57,22 +59,30 @@ def shouldPurgeList(value_node, key):
 
 def importRegistry(context):
 
-    logger = context.getLogger('plone.app.registry')
+    logger = context.getLogger("plone.app.registry")
     registry = queryUtility(IRegistry)
 
     if registry is None:
         logger.info("Cannot find registry")
         return
 
-    body = context.readDataFile('registry.xml')
-    if body is not None:
-        importer = RegistryImporter(registry, context)
-        importer.importDocument(body)
+    filepaths = ["registry.xml"]
+    if context.isDirectory("registry"):
+        for filename in context.listDirectory("registry"):
+            if not filename.startswith("."):
+                filepaths.append("registry/" + filename)
+
+    importer = RegistryImporter(registry, context)
+    for filepath in filepaths:
+        __traceback_info__ = filepath
+        body = context.readDataFile(filepath)
+        if body is not None:
+            importer.importDocument(body)
 
 
 def exportRegistry(context):
 
-    logger = context.getLogger('plone.app.registry')
+    logger = context.getLogger("plone.app.registry")
     registry = queryUtility(IRegistry)
 
     if registry is None:
@@ -82,14 +92,13 @@ def exportRegistry(context):
     exporter = RegistryExporter(registry, context)
     body = exporter.exportDocument()
     if body is not None:
-        context.writeDataFile('registry.xml', body, 'text/xml')
+        context.writeDataFile("registry.xml", safe_bytes(body), "text/xml")
 
 
-class RegistryImporter(object):
-    """Helper classt to import a registry file
-    """
+class RegistryImporter:
+    """Helper classt to import a registry file"""
 
-    LOGGER_ID = 'plone.app.registry'
+    LOGGER_ID = "plone.app.registry"
 
     def __init__(self, context, environ):
         self.context = context
@@ -102,56 +111,56 @@ class RegistryImporter(object):
         if self.environ.shouldPurge():
             self.context.records.clear()
 
-        i18n_domain = tree.attrib.get(ns('domain', I18N_NAMESPACE))
+        i18n_domain = tree.attrib.get(ns("domain", I18N_NAMESPACE))
         if i18n_domain:
             parseinfo.i18n_domain = i18n_domain
 
         for node in tree:
             if not isinstance(node.tag, str):
                 continue
-            condition = node.attrib.get('condition', None)
+            condition = node.attrib.get("condition", None)
             if condition and not evaluateCondition(condition):
                 continue
-            if node.tag.lower() == 'record':
+            if node.tag.lower() == "record":
                 self.importRecord(node)
-            elif node.tag.lower() == 'records':
+            elif node.tag.lower() == "records":
                 self.importRecords(node)
 
         parseinfo.i18n_domain = None
 
     def importRecord(self, node):
-        name = node.get('name', '')
-        if node.get('delete') is not None:
+        name = node.get("name", "")
+        if node.get("delete") is not None:
             self.logger.warning(
-                u"The 'delete' attribute of <record /> nodes is deprecated, "
-                u"it should be replaced with 'remove'."
+                "The 'delete' attribute of <record /> nodes is deprecated, "
+                "it should be replaced with 'remove'."
             )
-        remove = node.get('remove', node.get('delete', 'false'))
-        interfaceName = node.get('interface', None)
-        fieldName = node.get('field', None)
+        remove = node.get("remove", node.get("delete", "false"))
+        interfaceName = node.get("interface", None)
+        fieldName = node.get("field", None)
 
         if not name and (interfaceName and fieldName):
-            prefix = node.get('prefix', None)
+            prefix = node.get("prefix", None)
             if prefix is None:
                 prefix = interfaceName
 
-            name = "%s.%s" % (prefix, fieldName)
+            name = f"{prefix}.{fieldName}"
 
         if not name:
             raise NameError("No name given for <record /> node!")
 
         # Unicode is not supported
         name = str(name)
-        __traceback_info__ = 'record name: {0}'.format(name)
+        __traceback_info__ = f"record name: {name}"
 
         # Handle deletion and quit
-        if remove.lower() == 'true':
+        if remove.lower() == "true":
             if name in self.context.records:
                 del self.context.records[name]
                 self.logger.info("Removed record %s." % name)
             else:
                 self.logger.warning(
-                    "Record {0} was marked for deletion, but was not "
+                    "Record {} was marked for deletion, but was not "
                     "found.".format(name)
                 )
             return
@@ -171,25 +180,23 @@ class RegistryImporter(object):
                 field = IPersistentField(interface[fieldName])
             except ImportError:
                 self.logger.warning(
-                    'Failed to import interface {0} for '
-                    'record {1}'.format(interfaceName, name)
+                    "Failed to import interface {} for "
+                    "record {}".format(interfaceName, name)
                 )
                 interface = None
                 field = None
             except KeyError:
                 self.logger.warning(
-                    'Interface {0} specified for record %s has '
-                    'no field {1}.'.format(interfaceName, name, fieldName)
+                    "Interface {} specified for record %s has "
+                    "no field {}.".format(interfaceName, name, fieldName)
                 )
                 interface = None
                 field = None
             except TypeError:
                 self.logger.warning(
-                    "Field {0} in interface {1} specified for record {2} "
+                    "Field {} in interface {} specified for record {} "
                     "cannot be used as a persistent field.".format(
-                        fieldName,
-                        interfaceName,
-                        name
+                        fieldName, interfaceName, name
                     )
                 )
                 interface = None
@@ -203,44 +210,40 @@ class RegistryImporter(object):
         for child in node:
             if not isinstance(child.tag, str):
                 continue
-            elif child.tag.lower() == 'field':
+            elif child.tag.lower() == "field":
                 field_node = child
-            elif child.tag.lower() == 'value':
+            elif child.tag.lower() == "value":
                 value_node = child
 
         # Let field not potentially override interface[fieldName]
         if field_node is not None:
-            field_ref = field_node.attrib.get('ref', None)
+            field_ref = field_node.attrib.get("ref", None)
             if field_ref is not None:
                 # We have a field reference
                 if field_ref not in self.context:
                     raise KeyError(
-                        u'Record {0} references field for record {1}, '
-                        u'which does not exist'.format(name, field_ref)
+                        "Record {} references field for record {}, "
+                        "which does not exist".format(name, field_ref)
                     )
                 ref_record = self.context.records[field_ref]
                 field = FieldRef(field_ref, ref_record.field)
             else:
                 # We have a standard field
-                field_type = field_node.attrib.get('type', None)
+                field_type = field_node.attrib.get("type", None)
                 field_type_handler = queryUtility(
-                    IFieldExportImportHandler,
-                    name=field_type
+                    IFieldExportImportHandler, name=field_type
                 )
                 if field_type_handler is None:
                     raise TypeError(
-                        "Field of type {0} used for record {1} is not "
+                        "Field of type {} used for record {} is not "
                         "supported.".format(field_type, name)
                     )
                 else:
                     field = field_type_handler.read(field_node)
                     if not IPersistentField.providedBy(field):
                         raise TypeError(
-                            "Only persistent fields may be imported. {0} used "
-                            "for record {1} is invalid.".format(
-                                field_type,
-                                name
-                            )
+                            "Only persistent fields may be imported. {} used "
+                            "for record {} is invalid.".format(field_type, name)
                         )
 
         if field is not None and not IFieldRef.providedBy(field):
@@ -259,16 +262,14 @@ class RegistryImporter(object):
 
         if field is None:
             raise ValueError(
-                "Cannot find a field for the record {0}. Add a <field /> "
-                "element or reference an interface and field name.".format(
-                    name
-                )
+                "Cannot find a field for the record {}. Add a <field /> "
+                "element or reference an interface and field name.".format(name)
             )
 
         # Extract the value
 
         if value_node is not None:
-            value_purge = value_node.attrib.get('purge', '').lower() != 'false'
+            value_purge = value_node.attrib.get("purge", "").lower() != "false"
             value = elementToValue(field, value_node, default=_marker)
 
         # Now either construct or update the record
@@ -285,27 +286,29 @@ class RegistryImporter(object):
 
                 if not value_purge and type(value) == type(existing_value):
                     if isinstance(value, list):
-                        value = (
-                            existing_value +
-                            [v for v in value if v not in existing_value]
-                        )
+                        value = existing_value + [
+                            v for v in value if v not in existing_value
+                        ]
                     elif isinstance(value, tuple):
-                        value = (
-                            existing_value +
-                            tuple(
-                                [v for v in value if v not in existing_value]
-                            )
+                        value = existing_value + tuple(
+                            v for v in value if v not in existing_value
                         )
-                    elif isinstance(value, (set, frozenset, )):
+                    elif isinstance(
+                        value,
+                        (
+                            set,
+                            frozenset,
+                        ),
+                    ):
                         value = existing_value.union(value)
                     elif isinstance(value, dict):
                         for key, value in value.items():
                             # check if value is list, if so, let's add
                             # instead of overridding
                             if (
-                                type(value) == list and
-                                key in existing_value and
-                                not shouldPurgeList(value_node, key)
+                                type(value) == list
+                                and key in existing_value
+                                and not shouldPurgeList(value_node, key)
                             ):
                                 existing = existing_value[key]
                                 for item in existing:
@@ -328,26 +331,23 @@ class RegistryImporter(object):
         # May raise ImportError if interface can't be found or KeyError if
         # attribute is missing.
 
-        interfaceName = node.attrib.get('interface', None)
+        interfaceName = node.attrib.get("interface", None)
         if interfaceName is None:
-            raise KeyError(
-                u"A <records /> node must have an 'interface' attribute."
-            )
+            raise KeyError("A <records /> node must have an 'interface' attribute.")
 
         __traceback_info__ = "records name: " + interfaceName
 
         prefix = node.attrib.get(
-            'prefix',
-            None  # None means use interface.__identifier__
+            "prefix", None  # None means use interface.__identifier__
         )
 
-        if node.attrib.get('delete') is not None:
+        if node.attrib.get("delete") is not None:
             self.logger.warning(
-                u"The 'delete' attribute of <record /> nodes is deprecated, "
-                u"it should be replaced with 'remove'."
+                "The 'delete' attribute of <record /> nodes is deprecated, "
+                "it should be replaced with 'remove'."
             )
-        remove = node.attrib.get('remove', node.attrib.get('delete', 'false'))
-        remove = remove.lower() == 'true'
+        remove = node.attrib.get("remove", node.attrib.get("delete", "false"))
+        remove = remove.lower() == "true"
 
         # May raise ImportError
         interface = resolve(interfaceName)
@@ -359,10 +359,10 @@ class RegistryImporter(object):
         for child in node:
             if not isinstance(child.tag, str):
                 continue
-            elif child.tag.lower() == 'omit':
+            elif child.tag.lower() == "omit":
                 if child.text:
-                    omit.append(unicode(child.text))
-            elif child.tag.lower() == 'value':
+                    omit.append(safe_text(child.text))
+            elif child.tag.lower() == "value":
                 values.append(child)
 
         if remove and values:
@@ -375,15 +375,11 @@ class RegistryImporter(object):
                 if f in omit:
                     continue
 
-                child = etree.Element('value', key=f, purge='True')
+                child = etree.Element("value", key=f, purge="True")
                 values.append(child)
 
         # May raise TypeError
-        self.context.registerInterface(
-            interface,
-            omit=tuple(omit),
-            prefix=prefix
-        )
+        self.context.registerInterface(interface, omit=tuple(omit), prefix=prefix)
 
         if not values and not remove:
             # Skip out if there are no value records to handle
@@ -399,15 +395,15 @@ class RegistryImporter(object):
                 interface=interface.__identifier__,
                 field=value.attrib["key"],
                 prefix=prefix,
-                remove=repr(remove).lower()
+                remove=repr(remove).lower(),
             )
             field.append(value)
             self.importRecord(field)
 
 
-class RegistryExporter(object):
+class RegistryExporter:
 
-    LOGGER_ID = 'plone.app.registry'
+    LOGGER_ID = "plone.app.registry"
 
     def __init__(self, context, environ):
         self.context = context
@@ -415,7 +411,7 @@ class RegistryExporter(object):
         self.logger = environ.getLogger(self.LOGGER_ID)
 
     def exportDocument(self):
-        root = etree.Element('registry')
+        root = etree.Element("registry")
 
         for record in self.context.records.values():
             node = self.exportRecord(record)
@@ -425,43 +421,37 @@ class RegistryExporter(object):
 
     def exportRecord(self, record):
 
-        node = etree.Element('record')
-        node.attrib['name'] = record.__name__
+        node = etree.Element("record")
+        node.attrib["name"] = record.__name__
 
         if IInterfaceAwareRecord.providedBy(record):
-            node.attrib['interface'] = record.interfaceName
-            node.attrib['field'] = record.fieldName
+            node.attrib["interface"] = record.interfaceName
+            node.attrib["field"] = record.fieldName
 
         # write field
 
         field = record.field
         if IFieldRef.providedBy(field):
-            field_element = etree.Element('field')
-            field_element.attrib['ref'] = field.recordName
+            field_element = etree.Element("field")
+            field_element.attrib["ref"] = field.recordName
             node.append(field_element)
         else:
             field_type = IFieldNameExtractor(record.field)()
             handler = queryUtility(IFieldExportImportHandler, name=field_type)
             if handler is None:
                 self.logger.warning(
-                    'Field type {0} specified for record {1} '
-                    'cannot be exported'.format(field_type, record.__name__)
+                    "Field type {} specified for record {} "
+                    "cannot be exported".format(field_type, record.__name__)
                 )
             else:
                 field_element = handler.write(
-                    record.field,
-                    None,
-                    field_type,
-                    elementName='field'
+                    record.field, None, field_type, elementName="field"
                 )
                 node.append(field_element)
 
         # write value
         value_element = valueToElement(
-            record.field,
-            record.value,
-            name='value',
-            force=True
+            record.field, record.value, name="value", force=True
         )
         node.append(value_element)
 
